/* name readnet.c */

/* Copyright (C) Modicon, Inc. 1989,  All Rights Reserved. */

/*
readnet.c


This test program uses the SA85.SYS device driver to read the specified
Modbus Plus network, and display information about it, including the
personality of each node, token rotation time, etc..  The format for the
command line is as follows:

    READNET <routing path> </c>

where <routing path> is a valid Modbus Plus network address, and the optional
/c argument causes error counters to be cleared on all nodes.

*/

/*
include

*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netbios.h>
#include <netlib.h>

/*
prototypes

*/
int main( int argc, char *argv[] );
int read_station( void );
void show_node_type( int *pointer );

/*
global variables

*/
char path[ 80 ];                        /* path name to use             */
char path1[ 80 ];                       /* path name to use             */
char mbuffer[ 256 ];                    /* modbus message buffer        */
char active_stations[ 8 ];              /* map of active stations       */
NCB *nd;                                /* netbios control block ptr    */
NCB ncb_dummy;                          /* dummy for status command     */
int clear_flag = 0;                     /* nz to clear error counters   */

/*
main()


Parse the command line, and read information from the specified network.
*/
int main( argc, argv )
int argc;
char *argv[];
{
    char route[ 5 ];                    /* routing path                 */
    int routes = 0;                     /* assume no path given         */
    int peer_number;                    /* this node address            */
    char *mbptr;                        /* pointer to modbus buffer     */
    char *activeptr;                    /* pointer to active stations   */
    int i;

    printf( "READNET Modbus Plus Network Identification program. Version 1.0\n" );
    printf( "Copyright(C) 1989, Modicon, Inc.  All Rights Reserved.\n\n" );

    if( argc > 1 && ( ( strcmpi( argv[ 1 ], "/h" ) == 0 ) ||
                      ( strcmpi( argv[ 1 ], "-h" ) == 0 ) ||
                      ( strcmpi( argv[ 1 ], "?" ) == 0 ) ) ) {
        puts( "Usage is:" );
        puts( "    READNET <routing path> </c>" );
        puts( "    <routing path> is 0 to 5 byte routing path" );
        puts( "    </c> clears node error counters on specified network" ); 
        puts( "Examples:" );
        puts( "    READNET" );
        puts( "    READNET 1.2.3" );
        puts( "    READNET 1.2.3 /c" );
        exit( 1 );
    }

    if( argc > 2 && ( strcmpi( argv[ 2 ], "/c" ) == 0 ) )
        clear_flag = 1;                 /* clear error counters         */

    for( i = 0; i < 5; i++ )
        route[ i ] = 0;                 /* clear routing path           */

    if( argc > 1 ) {                    /* get routing info if present  */
        routes = sscanf( argv[ 1 ], "%d.%d.%d.%d.%d",
            &route[ 0 ],
            &route[ 1 ],
            &route[ 2 ],
            &route[ 3 ],
            &route[ 4 ] );              /* parse routing path argument  */
        for( i = 0; i < routes; i++ )  /* validate routing path        */
            if( route[ i ] < 1 || route[ i ] > 64 ) {
                puts( "ERROR, Routing value out of range." );
                exit( 1 );
            }
    }

    nd = &ncb_dummy;    
    nd->NCB_BUFFER = &mbuffer[ 0 ];
                                       
    if( ncb_status( nd, 0 ) != 0 ) {   /* check to see if board present */
        printf( "Modbus Plus board not found.\n" );
        exit( 1 );
    }
    else {
        peer_number = ( int ) mbuffer[ 6 ];
        printf( "Modbus Plus board identified as node address %d.\n",
            peer_number );
    }

    route[ routes ] = (char) peer_number;
    sprintf( path, "DM.%d.%d.%d.%d.%d", /* set up path string           */
        route[ 0 ],
        route[ 1 ],
        route[ 2 ],
        route[ 3 ],
        route[ 4 ] );
                                        /* open "path" on board 0       */
    if( ( nd = ncb_open( path, 0 ) ) == NULL ) {
        printf( "Unable to open DATA MASTER path.\n" );
        exit( 1 );
    }

    printf( "Path %c%c.%d.%d.%d.%d.%d opened.\n",
        nd->NCB_CALLNAME[0],
        nd->NCB_CALLNAME[1],
        nd->NCB_CALLNAME[2],
        nd->NCB_CALLNAME[3],
        nd->NCB_CALLNAME[4],
        nd->NCB_CALLNAME[5],
        nd->NCB_CALLNAME[6] );


    if( routes == 0 )                  /* if not path given            */
        nd->NCB_CALLNAME[ 2 ] = (char) peer_number;
    if( read_station() != 0 ) {
        printf( "Unable to communicate to Modbus Plus.\n" );
        ncb_close( nd );
        exit( 1 );
    }
    else {
        show_node_type( (int *) &mbuffer[ 7 ] );
    }
    if( mbuffer[ 7 ] == 4 ) {           /* if router node, new network  */
        routes++;
                                        /* other side of router         */
        nd->NCB_CALLNAME[ routes + 2 ] = mbuffer[ 12 ];

        if( read_station() != 0 ) {        
            printf( "Unable to communicate to Modbus Plus.\n" );
            ncb_close( nd );
            exit( 1 );
        }
        else {
            show_node_type( (int *) &mbuffer[ 7 ] );
        }
    }
                                            
    mbptr = mbuffer + 7 + ( 23 * 2 );   /* active stations              */
    activeptr = active_stations;
    for( i = 0; i < 8; i++ )
        activeptr[ i ] = mbptr[ i ];    /* copy active station table    */
    for( i = 0; i < 64; i++ ) {
        if( active_stations[ i / 8 ] &  /* if in active station table   */
            ( 1 << ( i % 8 ) ) ) {
            nd->NCB_CALLNAME[ routes + 2 ] = (char) i + 1;
            if( read_station() != 0 ) {
                printf( "Unable to read node %d.%d.%d.%d.%d\n",
                    nd->NCB_CALLNAME[2],
                    nd->NCB_CALLNAME[3],
                    nd->NCB_CALLNAME[4],
                    nd->NCB_CALLNAME[5],
                    nd->NCB_CALLNAME[6] );
            }
            else {
                show_node_type( (int *) &mbuffer[ 7 ] );
            }
        }
    }
    exit( 0 );
    return( 0 );
}

/*
read_station()


Issue the station management command to the specified station.
*/
int read_station() {
    mbuffer[ 1 ] = 0x8;                 /* interface diagnostic         */
    mbuffer[ 2 ] = 0x0;                 /* diag code high               */
    mbuffer[ 3 ] = 0x15;                /* station management           */
    mbuffer[ 4 ] = 0x0;                 /* diag. data high              */
    if( clear_flag )
        mbuffer[ 5 ] = 0x4;             /* read status, clear errors    */
    else
        mbuffer[ 5 ] = 0x3;             /* read status                  */
    if( ncb_send( nd, 6, mbuffer, 10 ) != 0 )
        return( -1 );                   /* return error code            */
    if( ncb_receive_wait( nd, mbuffer, 20 ) != 0 )
        return( -1 );                   /* return error code            */
    return( 0 );                        /* return success               */
}

/*
show_node_type()


Display the node type value.
*/
void show_node_type( pointer )
int *pointer;
{
    if( pointer[ 2 ] & 0xff00 )
        printf( "Station %d (%d) is ",  /* display both sides of router */
            pointer[ 2 ] & 0xff, pointer[ 2 ] >> 8 );
    else
        printf( "Station %d is ", pointer[ 2 ] );
    switch( pointer[ 0 ] ) {            /* display the node type        */
        case 1:
            printf( "a Controller, " );
            break;
        case 2:
            printf( "a Bridge MUX, " );
            break;
        case 3:
            printf( "a Host, " );
            break;
        case 4:
            printf( "a Bridge Plus, " );
            break;
        case 5:
            printf( "a Peer device, " );
            break;

        default:
            printf( "unknown (%d), ", pointer[ 0 ] );
            break;
    }
    printf( "Software version %d.%02d.\n",
        (pointer[ 1 ] >> 8) & 0xf,
        pointer[ 1 ] & 0xf );
}
